#include "qcirclelayout.h"
#include <QtMath>
#include <QDebug>
#include <QWidget>


QCircleLayout::QCircleLayout(QWidget *parent): QLayout(parent)
{
}

QCircleLayout::~QCircleLayout()
{
}

void QCircleLayout::setInitAngle(qreal angle)
{
  _initAngle = angle;
  //Заставляем компоновщик заново позиционировать элементы
  invalidate();
}

void QCircleLayout::setSparseCoef(qreal value)
{
  //Ограничение диапозона значений коэфициента разреженности
  if (value < 0.5)
    value = 0.5;
  if (value > 2)
    value = 2;
  _sparseCoef = value;
  //Заставляет компоновщик заново позиционировать элементы
  invalidate();
}

QSize QCircleLayout::sizeHint() const
{
  //Размеры по умолчанию пустого компоновщика
  if (count() < 1)
    return QSize(100, 100);
  qreal loSize = hintLoSize();
  //Рекомендованные размеры компоновщика
  return QSize(qRound(loSize), qRound(loSize));
}

void QCircleLayout::setGeometry(const QRect &rect)
{
  QLayout::setGeometry(rect);
  for(int i=0; i<count(); i++) {
    QSize size = itemAt(i)->sizeHint();    
    qreal angle = _initAngle + i*2*M_PI/count();
    //Координаты левого верхнего угла элемента в системе координат с центром в середине родительского виджета
    qreal xi = minLoRadius()*qCos(angle) - size.width()*0.5;
    qreal yi = -minLoRadius()*qSin(angle) - size.height()*0.5;
    //Позиция и размеры элемента в системе координат родительского виджета (с центром в левом верхнем углу)
    QRect geom(rect.center() + QPoint(qRound(xi), qRound(yi)), size);
    itemAt(i)->setGeometry(geom);
  }
}

QSize QCircleLayout::minimumSize() const
{
  //Рекомендованный размер компоновщика использован еще и как минимальный
  //это гарантирует, что все элементы будут видны
  return sizeHint();
}


qreal QCircleLayout::maxItemRadius() const
{
  qreal maxRadius = 0;
  for(int i=0; i<count(); i++) {
    QSize size = itemAt(i)->sizeHint();
    //Вычисление радиуса описанной окружности текущего элемента
    qreal curRadius = qSqrt(qPow(size.width(), 2) + qPow(size.height(), 2))/2;
    //Выбор максимального радиуса
    maxRadius = qMax(maxRadius, curRadius);
  }
  return maxRadius;
}

qreal QCircleLayout::minLoRadius() const
{
  //Если элементов 0 или 1, радиус равен нулю
  if (count() < 2)
    return 0;
  return _sparseCoef*maxItemRadius()/qSin(M_PI/count());
}

qreal QCircleLayout::hintLoSize() const
{
  return 2*minLoRadius() + 2*maxItemRadius();
}

void QCircleLayout::addItem(QLayoutItem *item)
{
  _items.append(item);
}

QLayoutItem *QCircleLayout::itemAt(int index) const
{
  //Проверка корректности индекса элемента (проверка обязательна)
  if (index < 0 || index >= count())
    return 0;
  return _items.at(index);
}

QLayoutItem *QCircleLayout::takeAt(int index)
{
  //Проверка корректности индекса элемента
  if (index < 0 || index >= count())
    return 0;
  return _items.takeAt(index);
}

int QCircleLayout::count() const
{
  return _items.count();
}
