#ifndef QCIRCLELAYOUT_H
#define QCIRCLELAYOUT_H

#include <QLayout>
#include <QLayoutItem>

//Компоновщик, располагающий элементы по кругу
//@author Lev Alekseevskiy
class QCircleLayout : public QLayout
{
    Q_OBJECT
    //Исходный угол от которого мы начинаем размещать компонуемые виджеты
    //по умолчанию = 0
    Q_PROPERTY(qreal initAngle READ initAngle WRITE setInitAngle)
    //Коэффициент разреженности (позволяет компоновать виджеты плотнее или свободнее)
    //по умолчанию = 1 (соответствует рекомендуемому значению и исключает наползание элементов друг на друга)
    Q_PROPERTY(qreal sparseCoef READ sparseCoef WRITE setSparseCoef)
public:
  QCircleLayout(QWidget * parent = 0);
  ~QCircleLayout();

  void setInitAngle(qreal angle);
  qreal initAngle(){ return _initAngle; }

  void setSparseCoef(qreal value);
  qreal sparseCoef() { return _sparseCoef; }

  // QLayoutItem interface
public:
  //Возвращает рекомендуемый размер компоновщика
  virtual QSize sizeHint() const;
  //Размещает компонуемые виджеты
  virtual void setGeometry(const QRect &rect);

  // QLayout interface
public:
  //Добавление нового элемента компоновки
  virtual void addItem(QLayoutItem *item);
  //Получение элемента компоновки по индексу
  virtual QLayoutItem *itemAt(int index) const;
  //Изъятие элемента компоновки по индексу
  virtual QLayoutItem *takeAt(int index);
  //Получение количества элементов компоновки
  virtual int count() const;
  //Устанавливает минимально допустимый размер компоновщика
  virtual QSize minimumSize() const;
private:
  //Список элементов компоновки
  QList<QLayoutItem*> _items;

  qreal _initAngle = 0;
  qreal _sparseCoef = 1;

  //Вычисление радиуса наибольшей описанной окружности вокруг элементов компоновки
  qreal maxItemRadius() const;
  //Вычисление радиуса окруженности, на которой позиционируется центр элемента компоновки
  qreal minLoRadius() const;
  //Оценка минимальных (рекомендуемых) габаритов компоновщика
  qreal hintLoSize() const;
};

#endif // QCIRCLELAYOUT_H
